﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WindowsFormsApplication2
{
    public partial class Form1 : Form
    {
        private List<PointCharge> charges = new List<PointCharge>();
        public Form1()
        {
            InitializeComponent();
            tSLProgress.Text = "Feldlinien werden gezeichnet";
            Application.DoEvents();
            charges.Add(new PointCharge(new Vect2D(pBElec.Width / 2, pBElec.Height / 2), 200));
        }
        float e0 = 10 ^ -12;
        const float pointSize = 15;
        int fieldLines = 20;
        CommandState commandState = CommandState.NoCommand;
        bool realMode = false;
        bool animationMode = false;
        enum CommandState
        {
            ConjugateCharge, PlaceCharge, ChangeCharge, DeleteCharge, NoCommand
        }
        private void pBElec_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;
            g.FillRectangle(Brushes.Black, 0, 0, pBElec.Width, pBElec.Height);
            for (int xPos = 0; xPos < pBElec.Width; xPos += 3)
            {
                for (int yPos = 0; yPos < pBElec.Height; yPos += 3)
                {
                    Vect2D fieldForce = GetFieldStrength(new Vect2D(xPos + 1, yPos + 1));
                    if (fieldForce.Length > 0 && fieldForce.Length < 25)
                    {
                        int blue = (int)((Math.Log(fieldForce.Length) + 8) * 100);
                        if (blue < 0)
                            blue = 0;
                        if (blue > 255)
                            blue = 255;
                        g.FillRectangle(new SolidBrush(Color.FromArgb(0, 0, blue)), xPos, yPos, 3, 3);
                    }
                }
            }


            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            Vect2D startOffset = new Vect2D(pointSize / 2, 0);
            foreach (PointCharge pc in charges)
            {
                if (realMode)
                    if (pc.Charge < 0)
                        continue;
                for (int cnt = fieldLines; cnt > 0; cnt--)
                {
                    startOffset.Angle = (float)((float)cnt / (float)fieldLines * 2 * Math.PI);
                    Vect2D curPos = pc.Position.GetCopy() + startOffset;
                    int autoKill = 10000;
                    while (autoKill > 0)
                    {
                        Vect2D nextP = GetFieldStrength(curPos);
                        if (nextP.Length > 1)
                            break;
                        nextP.Length = 5;
                        if (pc.Charge < 0)
                            nextP = curPos - nextP;
                        else
                            nextP += curPos;
                        g.DrawLine(Pens.LightBlue, curPos.GetPointF(), nextP.GetPointF());
                        curPos = nextP;
                        autoKill--;
                    }
                }
            }
            foreach (PointCharge pc in charges)
            {
                float halfPointSize = pointSize / 2;
                g.FillEllipse(Brushes.WhiteSmoke, pc.Position.X - halfPointSize, pc.Position.Y - halfPointSize, pointSize, pointSize);
                if (pc.Charge > 0)
                {
                    g.FillRectangle(Brushes.Green, pc.Position.X - 1, pc.Position.Y - halfPointSize + 2, 2, pointSize - 4);
                    g.FillRectangle(Brushes.Green, pc.Position.X - halfPointSize + 2, pc.Position.Y - 1, pointSize - 4, 2);
                }
                else
                    g.FillRectangle(Brushes.Red, pc.Position.X - halfPointSize + 2, pc.Position.Y - 1, pointSize - 4, 2);
            }
            tSLProgress.Text = "Bereit";
        }


        public Vect2D GetFieldStrength(Vect2D pos)
        {
            Vect2D vectSum = new Vect2D(0, 0);

            foreach (PointCharge pc in charges)
            {
                Vect2D distVect = pc.Position - pos;
                Vect2D fieldForceVect = distVect;
                float distance = -distVect.Length;
                fieldForceVect.Length = pc.Charge / (4 * (float)Math.PI * distance * distance * e0);

                vectSum += fieldForceVect;
            }

            return vectSum;
        }
        
        private void pBElec_MouseDown(object sender, MouseEventArgs e)
        {
            float halfPointSize = pointSize / 2;
            
            switch (commandState)
            {
                case CommandState.PlaceCharge:
                    
                    charges.Add(new PointCharge(new Vect2D(e.X, e.Y), 100));
                    RefreshFieldLines();
                    return;
            }
            foreach (PointCharge charge in charges)
            {
                if ((e.X > (charge.Position.X - halfPointSize)) && (e.X < (charge.Position.X + halfPointSize)))
                    if ((e.Y > (charge.Position.Y - halfPointSize)) && (e.Y < (charge.Position.Y + halfPointSize)))
                    {
                        switch (commandState)
                        {
                            case CommandState.ChangeCharge:
                                Form2 f2 = new Form2(charge.Charge);
                                f2.ShowDialog();
                                if (charge.Charge == f2.newCharge)
                                    return;

                                charge.Charge = f2.newCharge;
                                RefreshFieldLines();
                                break;
                            case CommandState.ConjugateCharge:
                                charge.Charge *= -1;
                                RefreshFieldLines();
                                break;
                            case CommandState.DeleteCharge:
                                charges.Remove(charge);
                                RefreshFieldLines();
                                break;
                                
                        }
                        return;
                    }

            }
        }
        private void RefreshFieldLines()
        {
            tSLProgress.Text = "Feldlinien werden gezeichnet";
            Application.DoEvents();
            if (animationMode)
                pBElec_Paint(pBElec, new PaintEventArgs(pBElec.CreateGraphics(), pBElec.ClientRectangle));
            else
                pBElec.Invalidate();
            
        }

        // command state and button control
        private void tSBChangeCharge_CheckedChanged(object sender, EventArgs e)
        {
            SetCommandState(tSBChangeCharge, CommandState.ChangeCharge);
        }
        private void tSPConjugateCharge_CheckedChanged(object sender, EventArgs e)
        {
            SetCommandState(tSPConjugateCharge, CommandState.ConjugateCharge);
        }

        private void tSBPlaceCharge_CheckedChanged(object sender, EventArgs e)
        {
            SetCommandState(tSBPlaceCharge, CommandState.PlaceCharge);
        }
        private void tSBDeleteCharge_CheckedChanged(object sender, EventArgs e)
        {
            SetCommandState(tSBDeleteCharge, CommandState.DeleteCharge);
        }

        private void SetCommandState(ToolStripButton button, CommandState state)
        {
            if (button.Checked)
            {
                UnCheckToolStripButtons(button);
                commandState = state;
            }
            else
                commandState = CommandState.NoCommand;
        }
        private void UnCheckToolStripButtons(ToolStripButton toSkip)
        {
            foreach (ToolStripItem tsi in toolStripControl.Items)
            {
                ToolStripButton button = tsi as ToolStripButton;
                if (button != null && !tsi.Equals(toSkip))
                    button.Checked = false;
            }
        }
        private void tSBTBFieldLines_TextChanged(object sender, EventArgs e)
        {
            if (!int.TryParse(tSTBFieldLines.Text, out fieldLines))
            {
                tSTBFieldLines.Text = "";
                tSTBFieldLines.Focus();
                return;
            }

        }

        private void tSTBFieldLines_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                
                e.SuppressKeyPress = true;
                e.Handled = true;
                RefreshFieldLines();
            }
        }

        private void checkBoxReal_CheckedChanged(object sender, EventArgs e)
        {
            realMode = !realMode;
            RefreshFieldLines();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            animationMode = !animationMode;
        }
    }
}